/*
 * FINDING THE MEDIAN
 * 
 * COMP 250 Assignment 2
 * Winter 2009
 * 
 */

import java.util.Random;
import java.util.Arrays;

/**
 * This class has methods that find the median of the numbers
 * contained in 2 sorted arrays.
 */
public class Median {
	
	// Random number generator used for generating random arrays.
	// You don't need to worry about this.
	public static Random rnd = new Random();
	
	// The 2 arrays that you must work with. They will both be
	// sorted individually.
	public static int[] A;
	public static int[] B;
	
	/**
	 * The main method automatically generates A and B, prints them,
	 * prints the median found using the given (slow) algorithm, and
	 * the median found using your recursive algorithm.
	 * 
	 * NOTE: Feel free to modify the N variable in this method. You shouldn't
	 * need to modify the rest of it, but go ahead if you'd like.
	 */
	public static void main (String[] args) {
		
		// N is the number of elements in each array.
		// NOTE: Feel free to modify this value to test your code with
		// smaller or bigger arrays!
		int N = 5;
		
		// Generate A and B (sorted arrays of randomly chosen elements)
		A = randomSortedArray(N);
		B = randomSortedArray(N);
		
		// Print A and B
		System.out.println("A = " + Arrays.toString(A));
		System.out.println("B = " + Arrays.toString(B));
		
		// Print the median, found using the slow algorithm.
		System.out.println("Median (slow) = " + slowMedian(0, N-1, 0, N-1));
		
		// Print the median, found using the fast algorithm you must implement
		System.out.println("Median (fast) = " + fastMedian(0, N-1, 0, N-1));
		
	}
	
	/**
	 * This method returns a new sorted array of the given size, whose elements
	 * are randomly generated.
	 * 
	 * NOTE: You don't need to worry about this method. Do not modify it.
	 */
	public static int[] randomSortedArray (int size) {
		
		int[] result = new int[size];
		
		int range = size * (2 + rnd.nextInt(4));
		int offset = size;
		
		for (int i = 0; i < size; i++) {
			result[i] = rnd.nextInt(range) - offset;
		}
		
		Arrays.sort(result);
		
		return result;
		
	}
	
	/**
	 * This method returns the median of the elements in two arrays. It runs
	 * in time O(n^2), where n is the size of one array.
	 * 
	 * The arguments define two sub-arrays of A and B, namely A[s,t] and B[p,q].
	 * Returned is the median of the elements in those sub-arrays. If we wanted to
	 * find the median of all elements in the entire A and B arrays, we would use:
	 *     slowMedian(0, N-1, 0, N-1).
	 * 
	 * NOTE: Do not modify this method. Feel free to look at it to understand
	 * what it does.
	 */
	public static int slowMedian (int s, int t, int p, int q) {

		// Number of elements in A and B respectively
		int a = t - s + 1;
		int b = q - p + 1;
		
		// Create a big temporary array
		int[] tmp = new int[a+b];
		
		// Copy all the elements of A and B to the big array. This is O(n).
		System.arraycopy(A, s, tmp, 0, a);
		System.arraycopy(B, p, tmp, a, b);
		
		// Sort the big array. The fastest comparison-based sorting time
		// is O(n*logn). This particular algorithm, however, is from the Java
		// library and has worst case O(n^2). We're using it for convenience.
		Arrays.sort(tmp);

		// Now the median is in the middle of tmp. If there is an even
		// number of elements, there are 2 middle elements; we take the lower
		// one. The following line finds the index we want.
		int mid = (a + b - 1) / 2;
		
		// Return the median.
		return tmp[mid];

	}
	
	/**
	 * This RECURSIVE method returns the median of the elements in two arrays.
	 * It runs in time O(log n), where n is the size of one array.
	 * 
	 * The arguments define two sub-arrays of A and B, namely A[s,t] and B[p,q].
	 * Returned is the median of the elements in those sub-arrays. If we wanted to
	 * find the median of all elements in the entire A and B arrays, we would use:
	 *     fastMedian(0, N-1, 0, N-1).
	 */
	public static int fastMedian (int s, int t, int p, int q) {

		/*
		 * TODO: WRITE THE CONTENTS OF THIS METHOD
		 * Remember, it must be recursive: fastMedian() must call itself!
		 */
		
		// For now, return a cool but meaningless number to avoid a compilation error
		return 250;

	}
	
}